==> kwy-wechatapi-1.4.2-725ba1.tar: src/openwx/common/WxBizMsgCrypt.php <==
<?php
namespace kwy\wechatapi\openwx\common;

/**
 * 对公众平台发送给公众账号的消息加解密示例代码.
 * @copyright Copyright (c) 1998-2014 Tencent Inc.
 */

/**
 * 1.第三方回复加密消息给公众平台；
 * 2.第三方收到公众平台发送的消息，验证消息的安全性，并对消息进行解密。
 */
class WxBizMsgCrypt
{
	private $token; //公众平台上，开发者设置的token
	private $encodingAesKey;    //公众平台上，开发者设置的EncodingAESKey
	private $appId; //公众平台的appId
    private $key;   //
    public static $block_size = 32;

    /**
     * 设置公众平台上，开发者设置的token
     * @author roc
     * @datetime:2019/12/17 11:24
     * @param $token
     * @return $this
     */
    public function setToken($token){
        $this->token = $token;
        return $this;
    }

    /**
     * 获取公众平台上，开发者设置的token
     * @author roc
     * @datetime:2019/12/17 11:24
     * @return string
     */
    public function getToken(){
        return $this->token;
    }

    /**
     * 设置公众平台上，开发者设置的EncodingAESKey
     * @author roc
     * @datetime:2019/12/17 11:24
     * @param $encodingAesKey
     * @return $this
     */
    public function setEncodingAesKey($encodingAesKey){
        $this->encodingAesKey = $encodingAesKey;
        $this->key = base64_decode($encodingAesKey. "=");
        return $this;
    }

    /**
     * 获取公众平台上，开发者设置的EncodingAESKey
     * @author roc
     * @datetime:2019/12/17 11:24
     * @return string
     */
    public function getEncodingAesKey(){
        return $this->encodingAesKey;
    }

    /**
     * 设置公众平台的appId
     * @author roc
     * @datetime:2019/12/17 11:24
     * @param $appid
     * @return $this
     */
    public function setAppId($appid){
        $this->appId = $appid;
        return $this;
    }

    /**
     * 获取公众平台的appId
     * @author roc
     * @datetime:2019/12/17 11:24
     * @return string
     */
    public function getAppId(){
        return $this->appId;
    }

    ###### 解密 start ######
    /**
     * 将公众平台回复用户的消息加密打包.
     * @author roc
     * @datetime:2019/12/23 14:17
     * <ol>
     *    <li>对要发送的消息进行AES-CBC加密</li>
     *    <li>生成安全签名</li>
     *    <li>将消息密文和安全签名打包成xml格式</li>
     * </ol>
     *
     * @param $replyMsg string 公众平台待回复用户的消息，xml格式的字符串
     * @param $timeStamp string 时间戳，可以自己生成，也可以用URL参数的timestamp
     * @param $nonce string 随机串，可以自己生成，也可以用URL参数的nonce
     * @param &$encryptMsg string 加密后的可以直接回复用户的密文，包括msg_signature, timestamp, nonce, encrypt的xml格式的字符串,
     *                      当return返回0时有效
     * @return int
     * @throws \Exception
     */
	public function encryptMsg($replyMsg, $timeStamp, $nonce, &$encryptMsg){
	    //加密
		$array = $this->encrypt($replyMsg, $this->appId);
		$ret = $array[0];
		if ($ret != 0) {
			return $ret;
		}

		if ($timeStamp == null) {
			$timeStamp = time();
		}
		$encrypt = $array[1];

		//生成安全签名
		$array = $this->getSha1($this->token, $timeStamp, $nonce, $encrypt);
		$ret = $array[0];
		if ($ret != 0) {
			return $ret;
		}
		$signature = $array[1];

		//生成发送的xml
		$xmlParse = new XmlParse();
		$encryptMsg = $xmlParse->generate($encrypt, $signature, $timeStamp, $nonce);
		return $encryptMsg;
	}

    /**
     * 检验消息的真实性，并且获取解密后的明文.
     * <ol>
     *    <li>利用收到的密文生成安全签名，进行签名验证</li>
     *    <li>若验证通过，则提取xml中的加密消息</li>
     *    <li>对消息进行解密</li>
     * </ol>
     *
     * @author roc
     * @datetime:2019/12/19 09:45
     * @param string $msgSignature 签名串，对应URL参数的msg_signature
     * @param null $timestamp 时间戳 对应URL参数的timestamp
     * @param string $nonce 随机串，对应URL参数的nonce
     * @param string $postData 密文，对应POST请求的数据
     * @return mixed 返回明文数据
     * @throws \Exception
     */
	public function decryptMsg($msgSignature, $timestamp = null, $nonce, $postData){
		if (strlen($this->encodingAesKey) != 43) {
			throw new \Exception("encodingAesKey 非法");
		}
		//提取密文
		$xmlParse = new XmlParse();
		$xmlResult = $xmlParse->extract($postData);

		if ($timestamp == null) {
			$timestamp = time();
		}

		$encrypt = $xmlResult[0];
		$touser_name = $xmlResult[1];

		//验证安全签名
        $signature = $this->getSha1($this->token, $timestamp, $nonce, $encrypt);
		if ($signature != $msgSignature) {
		    throw new \Exception("签名验证错误");
		}

        /**
         * 将密文解密成明文
         */
        $xml = $this->decrypt($encrypt, $this->appId);

        /**
         * 将xml转为array数组
         */
		$arr = $this->xml2Array($xml);
		return $arr;
	}

    /**
     * 解密密文
     * @author roc
     * @datetime:2019/12/18 17:04
     * @param string $encrypted 需要解密的密文
     * @param string $appid 微信公众号appid
     * @return bool|string 解密得到的明文
     * @throws \Exception
     */
    public function decrypt($encrypted, $appid)
    {

        try {

            $iv = substr($this->key, 0, 16);
            //使用BASE64对需要解密的字符串进行解码
            $decrypted = openssl_decrypt(base64_decode($encrypted), 'AES-256-CBC', $this->key, OPENSSL_RAW_DATA, $iv);
        } catch (\Exception $e) {
            throw new \Exception("解密失败");
        }

        try {
            //去除补位字符
            $result = $this->decode($decrypted);
            //去除16位随机字符串,网络字节序和AppId
            if (strlen($result) < 16) throw new \Exception("随机字符串小于16位");
            $content = substr($result, 16, strlen($result));
            $len_list = unpack("N", substr($content, 0, 4));

            $xml_len = $len_list[1];
            $xml_content = substr($content, 4, $xml_len);
            $from_appid = substr($content, $xml_len + 4);
        } catch (\Exception $e) {
            //print $e;
            throw new \Exception("解密后得到的buffer非法");
        }
        if ($from_appid != $appid){
            throw new \Exception("appid 校验错误");
        }
        return $xml_content;
    }

    /**
     * xml 转 array数组
     * @author roc
     * @datetime:2019/12/19 09:53
     * @param $xml
     * @return mixed
     */
    private function xml2Array($xml)
    {
        $obj = simplexml_load_string($xml, 'SimpleXMLElement', LIBXML_NOCDATA);
        return json_decode(json_encode($obj), 1);
    }

    /**
     * 对解密后的明文进行补位删除
     * @param decrypted 解密后的明文
     * @return 删除填充补位后的明文
     */
    public function decode($text)
    {
        $pad = ord(substr($text, -1));
        if ($pad < 1 || $pad > 32) {
            $pad = 0;
        }
        return substr($text, 0, (strlen($text) - $pad));
    }
    ###### 解密 end ######

    ###### 加密 start ######
    //提供基于PKCS7算法的加解密接口
    /**
     * 对明文进行加密
     * @author roc
     * @datetime:2019/12/23 14:25
     * @param string $text 需要加密的明文
     * @param string $appid 加密后的密文
     * @return string
     * @throws \Exception
     */
    public function encrypt($text, $appid)
    {
        try {
            //获得16位随机字符串，填充到明文之前
            $random = $this->getRandomStr();//"aaaabbbbccccdddd";
            $text = $random . pack("N", strlen($text)) . $text . $appid;
            $iv = substr($this->key, 0, 16);
            $text = $this->encode($text);
            $encrypted = openssl_encrypt($text,'AES-256-CBC',substr($this->key, 0, 32),OPENSSL_ZERO_PADDING,$iv);
            return $encrypted;
        } catch (\Exception $e) {
            //print $e;
            throw new \Exception("aes 加密失败");
        }
    }

    /**
     * 对需要加密的明文进行填充补位
     * @param string $text 需要进行填充补位操作的明文
     * @return string 补齐明文字符串
     */
    public function encode($text){
        $block_size = self::$block_size;
        $text_length = strlen($text);
        //计算需要填充的位数
        $amount_to_pad = $block_size - ($text_length % $block_size);
        if ($amount_to_pad == 0) {
            $amount_to_pad = $block_size;
        }
        //获得补位所用的字符
        $pad_chr = chr($amount_to_pad);
        $tmp = "";
        for ($index = 0; $index < $amount_to_pad; $index++) {
            $tmp .= $pad_chr;
        }
        return $text . $tmp;
    }

    /**
     * 随机生成16位字符串
     * @return string 生成的字符串
     */
    public function getRandomStr(){

        $str = "";
        $str_pol = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789abcdefghijklmnopqrstuvwxyz";
        $max = strlen($str_pol) - 1;
        for ($i = 0; $i < 16; $i++) {
            $str .= $str_pol[mt_rand(0, $max)];
        }
        return $str;
    }
    ###### 加密 end ######

    /**
     * 用SHA1算法生成安全签名
     * @author roc
     * @datetime:2019/12/19 09:56
     * @param string $token 票据
     * @param string $timestamp 时间戳
     * @param string $nonce 随机字符串
     * @param string $encrypt_msg 密文消息
     * @return string 签名数据
     * @throws \Exception
     */
    private function getSha1($token, $timestamp, $nonce, $encrypt_msg)
    {
        //排序
        try {
            $array = array($encrypt_msg, $token, $timestamp, $nonce);
            sort($array, SORT_STRING);
            $str = implode($array);
            return sha1($str);
        } catch (\Exception $e) {
            //print $e . "\n";
            throw new \Exception("sha加密生成签名失败");
        }
    }
}

==> kwy-wechatapi-1.6.6-807688.tar: src/openwx/common/WxBizMsgCrypt.php <==
<?php
namespace kwy\wechatapi\openwx\common;

/**
 * 对公众平台发送给公众账号的消息加解密示例代码.
 * @copyright Copyright (c) 1998-2014 Tencent Inc.
 */

/**
 * 1.第三方回复加密消息给公众平台；
 * 2.第三方收到公众平台发送的消息，验证消息的安全性，并对消息进行解密。
 */
class WxBizMsgCrypt
{
	private $token; //公众平台上，开发者设置的token
	private $encodingAesKey;    //公众平台上，开发者设置的EncodingAESKey
	private $appId; //公众平台的appId
    private $key;   //
    public static $block_size = 32;

    /**
     * 设置公众平台上，开发者设置的token
     * @author roc
     * @datetime:2019/12/17 11:24
     * @param $token
     * @return $this
     */
    public function setToken($token){
        $this->token = $token;
        return $this;
    }

    /**
     * 获取公众平台上，开发者设置的token
     * @author roc
     * @datetime:2019/12/17 11:24
     * @return string
     */
    public function getToken(){
        return $this->token;
    }

    /**
     * 设置公众平台上，开发者设置的EncodingAESKey
     * @author roc
     * @datetime:2019/12/17 11:24
     * @param $encodingAesKey
     * @return $this
     */
    public function setEncodingAesKey($encodingAesKey){
        $this->encodingAesKey = $encodingAesKey;
        $this->key = base64_decode($encodingAesKey. "=");
        return $this;
    }

    /**
     * 获取公众平台上，开发者设置的EncodingAESKey
     * @author roc
     * @datetime:2019/12/17 11:24
     * @return string
     */
    public function getEncodingAesKey(){
        return $this->encodingAesKey;
    }

    /**
     * 设置公众平台的appId
     * @author roc
     * @datetime:2019/12/17 11:24
     * @param $appid
     * @return $this
     */
    public function setAppId($appid){
        $this->appId = $appid;
        return $this;
    }

    /**
     * 获取公众平台的appId
     * @author roc
     * @datetime:2019/12/17 11:24
     * @return string
     */
    public function getAppId(){
        return $this->appId;
    }

    ###### 解密 start ######
    /**
     * 将公众平台回复用户的消息加密打包.
     * @author roc
     * @datetime:2019/12/23 14:17
     * <ol>
     *    <li>对要发送的消息进行AES-CBC加密</li>
     *    <li>生成安全签名</li>
     *    <li>将消息密文和安全签名打包成xml格式</li>
     * </ol>
     *
     * @param $replyMsg string 公众平台待回复用户的消息，xml格式的字符串
     * @param $timeStamp string 时间戳，可以自己生成，也可以用URL参数的timestamp
     * @param $nonce string 随机串，可以自己生成，也可以用URL参数的nonce
     * @return int
     * @throws \Exception
     */
	public function encryptMsg($replyMsg, $timeStamp, $nonce){
	    //加密
        $encrypt = $this->encrypt($replyMsg, $this->appId);
		//生成安全签名
        $signature = $this->getSha1($this->token, $timeStamp, $nonce, $encrypt);

		//生成发送的xml
		$xmlParse = new XmlParse();
		$encryptMsg = $xmlParse->generate($encrypt, $signature, $timeStamp, $nonce);
		return $encryptMsg;
	}

    /**
     * 检验消息的真实性，并且获取解密后的明文.
     * <ol>
     *    <li>利用收到的密文生成安全签名，进行签名验证</li>
     *    <li>若验证通过，则提取xml中的加密消息</li>
     *    <li>对消息进行解密</li>
     * </ol>
     *
     * @author roc
     * @datetime:2019/12/19 09:45
     * @param string $msgSignature 签名串，对应URL参数的msg_signature
     * @param null $timestamp 时间戳 对应URL参数的timestamp
     * @param string $nonce 随机串，对应URL参数的nonce
     * @param string $postData 密文，对应POST请求的数据
     * @return mixed 返回明文数据
     * @throws \Exception
     */
	public function decryptMsg($msgSignature, $timestamp = null, $nonce, $postData){
		if (strlen($this->encodingAesKey) != 43) {
			throw new \Exception("encodingAesKey 非法");
		}
		//提取密文
		$xmlParse = new XmlParse();
		$xmlResult = $xmlParse->extract($postData);

		if ($timestamp == null) {
			$timestamp = time();
		}

		$encrypt = $xmlResult[0];
		$touser_name = $xmlResult[1];

		//验证安全签名
        $signature = $this->getSha1($this->token, $timestamp, $nonce, $encrypt);
		if ($signature != $msgSignature) {
		    throw new \Exception("签名验证错误");
		}

        /**
         * 将密文解密成明文
         */
        $xml = $this->decrypt($encrypt, $this->appId);

        /**
         * 将xml转为array数组
         */
		$arr = $this->xml2Array($xml);
		return $arr;
	}

    /**
     * 解密密文
     * @author roc
     * @datetime:2019/12/18 17:04
     * @param string $encrypted 需要解密的密文
     * @param string $appid 微信公众号appid
     * @return bool|string 解密得到的明文
     * @throws \Exception
     */
    public function decrypt($encrypted, $appid)
    {

        try {

            $iv = substr($this->key, 0, 16);
            //使用BASE64对需要解密的字符串进行解码
            $decrypted = openssl_decrypt(base64_decode($encrypted), 'AES-256-CBC', $this->key, OPENSSL_RAW_DATA, $iv);
        } catch (\Exception $e) {
            throw new \Exception("解密失败");
        }

        try {
            //去除补位字符
            $result = $this->decode($decrypted);
            //去除16位随机字符串,网络字节序和AppId
            if (strlen($result) < 16) throw new \Exception("随机字符串小于16位");
            $content = substr($result, 16, strlen($result));
            $len_list = unpack("N", substr($content, 0, 4));

            $xml_len = $len_list[1];
            $xml_content = substr($content, 4, $xml_len);
            $from_appid = substr($content, $xml_len + 4);
        } catch (\Exception $e) {
            //print $e;
            throw new \Exception("解密后得到的buffer非法");
        }
        if ($from_appid != $appid){
            throw new \Exception("appid 校验错误");
        }
        return $xml_content;
    }



    /**
     * 小程序 加密数据解密算法
     * @author roc
     * @datetime:2020/2/4 01:18
     * @param string $appid 小程序appid
     * @param string $sessionKey 会话秘钥
     * @param string $encryptedData 包括敏感数据在内的完整用户信息的加密数据
     * @param string $iv 会话密钥
     * @return string
     * @throws \Exception
     */
    public function decryptData($appid,$sessionKey,$encryptedData,$iv)
    {
        if (strlen($sessionKey) != 24) {
            throw new \Exception("非法的Aes密钥");
        }
        $aesKey = base64_decode($sessionKey);

        if (strlen($iv) != 24) {
            throw new \Exception("非法的iv");
        }
        $aesIV=base64_decode($iv);

        $aesCipher=base64_decode($encryptedData);

        $result = openssl_decrypt( $aesCipher, "AES-128-CBC", $aesKey, 1, $aesIV);

        $dataObj = json_decode($result);

        if( $dataObj  == NULL ){
            throw new \Exception("解密后得到的buffer非法");
        }
        if( $dataObj->watermark->appid != $appid )
        {
            throw new \Exception("解密后得到的buffer非法2");
        }
        return $result;
    }

    /**
     * xml 转 array数组
     * @author roc
     * @datetime:2019/12/19 09:53
     * @param $xml
     * @return mixed
     */
    private function xml2Array($xml)
    {
        $obj = simplexml_load_string($xml, 'SimpleXMLElement', LIBXML_NOCDATA);
        return json_decode(json_encode($obj), 1);
    }

    /**
     * 对解密后的明文进行补位删除
     * @param decrypted 解密后的明文
     * @return 删除填充补位后的明文
     */
    public function decode($text)
    {
        $pad = ord(substr($text, -1));
        if ($pad < 1 || $pad > 32) {
            $pad = 0;
        }
        return substr($text, 0, (strlen($text) - $pad));
    }
    ###### 解密 end ######

    ###### 加密 start ######
    //提供基于PKCS7算法的加解密接口
    /**
     * 对明文进行加密
     * @author roc
     * @datetime:2019/12/23 14:25
     * @param string $text 需要加密的明文
     * @param string $appid 加密后的密文
     * @return string
     * @throws \Exception
     */
    public function encrypt($text, $appid)
    {
        try {
            //获得16位随机字符串，填充到明文之前
            $random = $this->getRandomStr();//"aaaabbbbccccdddd";
            $text = $random . pack("N", strlen($text)) . $text . $appid;
            $iv = substr($this->key, 0, 16);
            $text = $this->encode($text);
            $encrypted = openssl_encrypt($text,'AES-256-CBC',substr($this->key, 0, 32),OPENSSL_ZERO_PADDING,$iv);
            return $encrypted;
        } catch (\Exception $e) {
            //print $e;
            throw new \Exception("aes 加密失败");
        }
    }

    /**
     * 对需要加密的明文进行填充补位
     * @param string $text 需要进行填充补位操作的明文
     * @return string 补齐明文字符串
     */
    public function encode($text){
        $block_size = self::$block_size;
        $text_length = strlen($text);
        //计算需要填充的位数
        $amount_to_pad = $block_size - ($text_length % $block_size);
        if ($amount_to_pad == 0) {
            $amount_to_pad = $block_size;
        }
        //获得补位所用的字符
        $pad_chr = chr($amount_to_pad);
        $tmp = "";
        for ($index = 0; $index < $amount_to_pad; $index++) {
            $tmp .= $pad_chr;
        }
        return $text . $tmp;
    }

    /**
     * 随机生成16位字符串
     * @return string 生成的字符串
     */
    public function getRandomStr(){

        $str = "";
        $str_pol = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789abcdefghijklmnopqrstuvwxyz";
        $max = strlen($str_pol) - 1;
        for ($i = 0; $i < 16; $i++) {
            $str .= $str_pol[mt_rand(0, $max)];
        }
        return $str;
    }
    ###### 加密 end ######

    /**
     * 用SHA1算法生成安全签名
     * @author roc
     * @datetime:2019/12/19 09:56
     * @param string $token 票据
     * @param string $timestamp 时间戳
     * @param string $nonce 随机字符串
     * @param string $encrypt_msg 密文消息
     * @return string 签名数据
     * @throws \Exception
     */
    private function getSha1($token, $timestamp, $nonce, $encrypt_msg)
    {
        //排序
        try {
            $array = array($encrypt_msg, $token, $timestamp, $nonce);
            sort($array, SORT_STRING);
            $str = implode($array);
            return sha1($str);
        } catch (\Exception $e) {
            //print $e . "\n";
            throw new \Exception("sha加密生成签名失败");
        }
    }
}
